/*
 * hiddencontrollersdialog.cpp
 * ---------------------------
 * Begin
 *   2016-01-10
 * 
 * Copyright
 *   (C) 2016 Guido Scholz <guido-scholz@gmx.net>
 * 
 * Description
 *   Dialog window to display hidden controllers
 *
 * License
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 */


#include <QDialog>
#include <QVBoxLayout>
#include <QDialogButtonBox>

#include "hiddencontrollersdialog.h"



HiddenControllersDialog:: HiddenControllersDialog(QWidget* parent):
 QDialog(parent)
{
    setModal(false);
    setObjectName("HiddenControllersDialog");
    setWindowTitle(tr("Show hidden controllers"));

    QVBoxLayout* baseLayout = new QVBoxLayout();

    listWidget = new QListWidget();
    listWidget->setSelectionMode(QAbstractItemView::SingleSelection);
    listWidget->setMinimumHeight(100);
    listWidget->setMinimumWidth(80);


    // show button
    showButton = new QPushButton(tr("&Show"));
    showButton->setAutoDefault(false);
    connect(showButton, SIGNAL(pressed()),
            this, SLOT(slotShowSelectedController()));

    // OK button
    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Close);
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(accept()));

    buttonBox->addButton(showButton, QDialogButtonBox::ActionRole);

    baseLayout->addWidget(listWidget);
    baseLayout->addWidget(buttonBox);
    setLayout(baseLayout);
}


void HiddenControllersDialog::setControllersList(
        const QList<hiddenitem>& hlist)
{
    for (int i = 0; i < hlist.size(); ++i) {

        QListWidgetItem *item = new QListWidgetItem(
                hlist.at(i).listtext, listWidget);

        item->setData(Qt::UserRole, hlist.at(i).id);
    }
    showButton->setEnabled(listWidget->count() > 0);
}


void HiddenControllersDialog::slotShowSelectedController()
{
    int cid;

    if (listWidget->count() == 0)
        return;

    QListWidgetItem *item = listWidget->currentItem();
    if (item != NULL)  {
        cid = item->data(Qt::UserRole).toInt();
        delete listWidget->takeItem(listWidget->row(item));
        if (listWidget->count() == 0)
            showButton->setEnabled(false);

        emit showController(cid);
    }
}

