/***************************************************************************
                               LocoDialog.h
                             -----------------
    begin                : 11.11.2000
    copyright            : (C) 2000 by Markus Pfeiffer
    email                : mail@markus-pfeiffer.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LOCODIALOG_H
#define LOCODIALOG_H

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qfont.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlcdnumber.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qstring.h>
#include <qtabwidget.h>
#include <qtextstream.h>

#ifdef WIN32
#include "config_w32.h"
#else
#include "config.h"
#endif

// where to find the icons, decoders and protocols
// #define RESOURCE_DIR    PREFIX "/share/" PACKAGE "/"
#define PIX_DIR         RESOURCE_DIR "/"
#define LOCO_PIX_DIR    RESOURCE_DIR "/locopixmaps/"
#define DECODER_FILE 	RESOURCE_DIR "/decoders"
#define PROTOCOL_FILE   RESOURCE_DIR "/protocols"


enum buttonTypes
{
    PUSHBUTTON,
    SWITCH
};


struct Decoder {
    Decoder(QString _name, unsigned int fc, QString _protocols)
    { name = _name; functions = fc; protocols = _protocols; }
    QString name;
    unsigned int functions;
    QString protocols;
};

struct Protocol{
    Protocol(QString _name, unsigned int _steps, unsigned int ar)
    {name= _name; steps= _steps; addressrange= ar;}
    QString name;
    unsigned int steps;
    unsigned int addressrange;
};

enum fType {
    ftLight = 0, ftSmoke, ftCoupl, ftSound, ftShunt, ftCabinlight,
    ftWagonlight, ftGearlight, ftWhistle, ftBell, ftInspectorwhistle,
    ftRearlight, ftPantographl, ftPantographr, ftBreaking
};

struct FunktionTypes {
    int ftId;
    const char* ftName;
    const char* ftIcon;
};

enum FunktionTypeCount {
    functiontypes = 15
};

/*maximum number of function buttons*/
enum {
    maxfbuttoncount = 6
};

static const FunktionTypes ftData[functiontypes] = {
    {ftLight, QT_TRANSLATE_NOOP("LocoDialog", "Light"), "light"},
    {ftSmoke, QT_TRANSLATE_NOOP("LocoDialog", "Smoke"), "smoke"},
    {ftCoupl, QT_TRANSLATE_NOOP("LocoDialog", "Coupling"), "coupl"},
    {ftSound, QT_TRANSLATE_NOOP("LocoDialog", "Sound"), "sound"},
    {ftShunt, QT_TRANSLATE_NOOP("LocoDialog", "Shunting"), "shunt"},
    {ftCabinlight, QT_TRANSLATE_NOOP("LocoDialog", "Cabinlight"), "cabinlight"},
    {ftWagonlight, QT_TRANSLATE_NOOP("LocoDialog", "Wagonlight"), "wagonlight"},
    {ftGearlight, QT_TRANSLATE_NOOP("LocoDialog", "Gearlight"), "gearlight"},
    {ftWhistle, QT_TRANSLATE_NOOP("LocoDialog", "Whistle"), "whistle"},
    {ftBell, QT_TRANSLATE_NOOP("LocoDialog", "Bell"), "bell"},
    {ftInspectorwhistle, QT_TRANSLATE_NOOP("LocoDialog", "Inspector whistle"),
        "inspectorwhistle"},
    {ftRearlight, QT_TRANSLATE_NOOP("LocoDialog", "Rear light"), "rearlight"},
    {ftPantographl, QT_TRANSLATE_NOOP("LocoDialog", "Pantograph 1"),
        "pantographl"},
    {ftPantographr, QT_TRANSLATE_NOOP("LocoDialog", "Pantograph 2"),
        "pantographr"},
    {ftBreaking, QT_TRANSLATE_NOOP("LocoDialog", "Breaking"), "breaking"}
};
// TODO: add
// ("bright light");
// ("breaking");
// ("turbines"); Lüfter
// Sound ein
// Maschinenraum-Beleuchtung

struct Function {
    Function(bool v, bool a, int bt, int ft) {
        visible = v;
        active = a;
        buttontype = bt;
        functiontype = ft;
    }
    bool visible; bool active; int buttontype; int functiontype;
};

// structure to collect function settings widgets
struct FunctionWidgets {
    FunctionWidgets(QHBoxLayout* l, QLabel* fl, QCheckBox* v,
            QLabel* tl, QComboBox* i, QLabel*al, QComboBox* b) {
        lineLayout = l;
        fLabel = fl;
        visibleCB = v;
        typeLabel = tl;
        functiontypeCB = i;
        activateLabel = al;
        buttontypeCB = b;
    }
    QHBoxLayout* lineLayout;
    QLabel* fLabel;
    QCheckBox* visibleCB;
    QLabel* typeLabel;
    QComboBox* functiontypeCB;
    QLabel* activateLabel;
    QComboBox* buttontypeCB;
};


class LocoDialog: public QDialog
{
    Q_OBJECT

    int          maxAddress;

    QList<Decoder> decoders;
    QList<Protocol> protocols;
    QList<FunctionWidgets> functionwidgets;

    QLineEdit    *leName;
    QLineEdit    *leAlias;
    QSpinBox     *spAddress;
    QLineEdit    *locoIcon;
    QLineEdit    *leRealMaxSpeed;
    QSlider      *sliderMaxSpeed;
    QSlider      *sliderAvgSpeed;
    QSlider      *sliderMinSpeed;
    QSlider      *sliderAccelTime;
    QSlider      *sliderBreakTime;
    QVBoxLayout  *functionLayout;
    QComboBox    *decoderCB;
    QComboBox    *protocolCB;
    QCheckBox	 *cbReverseDirection;
    QRadioButton *rbUnitkm;
    QTabWidget   *tabs;
    QLabel	 *labelDescription;
    QCheckBox	 *keepLimitCB;
    QCheckBox	 *followLimitCB;

    void setupTabGeneral();
    void setupTabFuncs();
    void setupTabSpeeds();
    void setupTabCrcf();
    void updateProtocolValues(const Protocol&);
    bool loadDecoderFile();
    bool loadProtocolFile();
    void printDescription(const Protocol&);
    void updateFunctionView(int);
    void addFunctionLine(int);
    void removeFunctionLine();

public:
    LocoDialog(QWidget* parent = 0);

    int exec();
    void setLocoName(const QString&);
    QString getLocoName() const;
    void setTrainNumber(unsigned int);
    unsigned int getTrainNumber();
    void setLocoPixmapName(const QString&);
    QString getLocoPixmapName() const;
    void setReverseDirection(bool);
    bool getReverseDirection();
    void setDecoder(const QString&, const QString&);
    void getDecoder(QString&, QString&);
    void setAddress(int);
    int getAddress();
    void setAttenuationTimes(int, int);
    void getAttenuationTimes(int&, int&);
    void setLocoSpeeds(int, int, int, int);
    void getLocoSpeeds(int&, int&, int&, int&);
    void setSpeedUnitKmh(bool);
    bool getSpeedUnitKmh();
    void setFunctions(const QList<Function>&);
    void getFunctions(QList<Function>&);
    void setKeepLimit(bool);
    bool getKeepLimit();
    void setFollowLimit(bool);
    bool getFollowLimit();

private slots:
    void verify();
    void slotGetIcon();
    void updateProtocolData(const QString&);
    void updateProtocolDescription(const QString&);
    void updateProtocolCB(int);
    void visibilityChanged(bool);
};

#endif    //LOCODIALOG_H
