/***************************************************************************
                               preferencesdlg.cpp
                             -------------------
    begin                : 2004-10-26
    copyright            : (C) 2004-2008 by Guido Scholz
    email                : guido-scholz@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QFileDialog>
#include <QDialogButtonBox>

#include "preferencesdlg.h"


PreferencesDialog::PreferencesDialog(QWidget* parent): QDialog(parent)
{
    setModal(true);
    setWindowTitle(tr("Preferences"));
    QVBoxLayout* mainLayout = new QVBoxLayout();
    mainLayout->setObjectName("mainLayout");

    // file groupbox
    QGroupBox *ExtPreferencesGB = 
        new QGroupBox(tr("Preferences"), this);

    QVBoxLayout *extGBLayout = new QVBoxLayout();
    extGBLayout->setObjectName("extGBLayout");
    ExtPreferencesGB->setLayout(extGBLayout);

    // line for autoload option
    QHBoxLayout *fileLayout = new QHBoxLayout();
    fileLayout->setObjectName("fileLayout");
    extGBLayout->addLayout(fileLayout);

    autoloadCB = new QCheckBox(tr("&Autoload file on startup"),
            ExtPreferencesGB);
    autoloadCB->setObjectName("autoloadCB");
    fileLayout->addWidget(autoloadCB);
    connect(autoloadCB, SIGNAL(toggled(bool)),
            this, SLOT(slotAutoloadChanged(bool)));

    fileLayout->addStretch();

    filePB = new QPushButton(tr("&Choose..."), ExtPreferencesGB);
    fileLayout->addWidget(filePB);
    connect(filePB, SIGNAL(clicked()), this, SLOT(chooseFile()));

    // line for autoload filename
    fileLE = new QLineEdit(ExtPreferencesGB);
    fileLE->setObjectName("fileLE");
    fileLE->setMinimumWidth(180);
    extGBLayout->addWidget(fileLE);

    // line for browser option
    QHBoxLayout *browserLayout = new QHBoxLayout();
    browserLayout->setObjectName("browserLayout");
    extGBLayout->addLayout(browserLayout);

    QLabel *browserLabel = new QLabel(tr("&Browser for online help"));
    browserLayout->addWidget(browserLabel);
    browserLayout->addStretch();

    browserCB = new QComboBox(this);
    browserCB->setObjectName("browserCB");
    browserCB->addItem("Firefox");   /*0 */
    browserCB->addItem("Konqueror"); /*1 */
    browserCB->addItem("Mozilla");   /*2 */
    browserCB->addItem("Netscape");  /*3 */
    browserCB->addItem("Opera");     /*4 */
    browserLayout->addWidget(browserCB);
    browserLabel->setBuddy(browserCB);

    // line for scale option
    QHBoxLayout *scaleLayout = new QHBoxLayout();
    scaleLayout->setObjectName("scaleLayout");
    extGBLayout->addLayout(scaleLayout);

    QLabel *scaleLabel = new QLabel(tr("&Rail transport modelling scale"));
    scaleLayout->addWidget(scaleLabel);
    scaleLayout->addStretch();

    scaleCB = new QComboBox(this);
    scaleCB->setObjectName("scaleCB");
    scaleCB->addItem("2 (1:22,5)");  /*0 */
    scaleCB->addItem("1 (1:32)");    /*1 */
    scaleCB->addItem("0 (1:45)");    /*2 */
    scaleCB->addItem("S (1:64)");    /*3 */
    scaleCB->addItem("H0 (1:87)");   /*4 */
    scaleCB->addItem("TT (1:120)");  /*5 */
    scaleCB->addItem("N (1:160)");   /*6 */
    scaleCB->addItem("Z (1:220)");   /*7 */
    scaleLayout->addWidget(scaleCB);
    scaleLabel->setBuddy(scaleCB);

    // OK and Cancel buttons
    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok |
            QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this,
            SLOT(reject()));

    //arrange layout
    mainLayout->addWidget(ExtPreferencesGB);
    mainLayout->addWidget(buttonBox);

    setLayout(mainLayout);
}


void PreferencesDialog::chooseFile()
{
    QString filename = QFileDialog::getOpenFileName(this, tr("Choose file"),
            QDir::homePath(), tr("Locomotive controller sets") +
            " (*" + DTCLFILEEXT + ")");

    if (filename != NULL)
        fileLE->setText(filename);
}


void PreferencesDialog::setDialogData(bool autoload,
        const QString& filename, int browserno, int scale)
{
    autoloadCB->setChecked(autoload);
    fileLE->setText(filename);
    browserCB->setCurrentIndex(browserno);
    scaleCB->setCurrentIndex(scale);
    slotAutoloadChanged(autoload);
}


bool PreferencesDialog::getAutoload()
{
    return autoloadCB->isChecked();
}


QString PreferencesDialog::getAutoloadFile()
{
    return fileLE->text();
}


int PreferencesDialog::getBrowserNr()
{
    return browserCB->currentIndex();
}

int PreferencesDialog::getScale()
{
    return scaleCB->currentIndex();
}

void PreferencesDialog::slotAutoloadChanged(bool autoenabled)
{
    filePB->setEnabled(autoenabled);
    fileLE->setEnabled(autoenabled);
}

