/***************************************************************************
 srcp_test.cpp
 -------------
 Copyright    : (C) 2008-2016 Guido Scholz
 E-Mail       : guido.scholz@bayernline.de
 Begin        : 16.12.2008
 Last modified: $Date: 2008/12/31 07:39:01 $
                $Revision: 1.2 $
***************************************************************************/

/**************************************************************************
 *                                                                        *
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 2 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 **************************************************************************/

/**************************************************************************
   test program to check valid SRCP string translations by "make check"
 **************************************************************************/


#include <QApplication>
#include <QString>

#include "srcpmessage.h"


bool checkSrcp08String(const QString& nominal, SrcpMessage* sm)
{
    QString result = sm->getSrcpMessageStr();
    bool isOK = (nominal == result);
    if (!isOK)
        qWarning("SRCP 08 Failed: Nominal= \"%s\", Result= \"%s\"",
                nominal.toLatin1().data(), result.toLatin1().data());
    return isOK;
}


bool runSrcpTest()
{
    QString nominal, result;
    bool returnvalue = true;


    /*SRCP Description*/
    qWarning("Testing SRCP DESCRIPTION messages...");

    qWarning("  GET...");
    /* DESCRIPTION GET*/
    DescriptionMessage descriptionm =
        DescriptionMessage(SrcpMessage::mtDescriptionGet, (unsigned int)3);

    if (!checkSrcp08String("GET 3 DESCRIPTION", &descriptionm))
        returnvalue = false;

    /* DESCRIPTION INFO*/
    qWarning("  INFO...");

    /* FB*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devFb);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION FB", &descriptionm))
        returnvalue = false;

    /* GA*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devGa);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION GA", &descriptionm))
        returnvalue = false;

    /* GL*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devGl);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION GL", &descriptionm))
        returnvalue = false;

    /* GM*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devGm);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION GM", &descriptionm))
        returnvalue = false;

    /* LOCK*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devLock);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION LOCK", &descriptionm))
        returnvalue = false;

    /* POWER*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devPower);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION POWER", &descriptionm))
        returnvalue = false;

    /* SERVER*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devServer);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION SERVER", &descriptionm))
        returnvalue = false;

    /* SESSION*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devSession);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION SESSION", &descriptionm))
        returnvalue = false;

    /* SM*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devSm);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION SM", &descriptionm))
        returnvalue = false;

    /* TIME*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3, SrcpMessage::devTime);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION TIME", &descriptionm))
        returnvalue = false;

    /* SERVER SESSION TIME*/
    descriptionm = DescriptionMessage(SrcpMessage::mtDescriptionInfo,
            SrcpMessage::code100, (unsigned int)3,
            SrcpMessage::devServer | SrcpMessage::devSession
            | SrcpMessage::devTime);
    
    if (!checkSrcp08String("100 INFO 3 DESCRIPTION SERVER SESSION TIME",
                &descriptionm))
        returnvalue = false;


 
    /*SRCP Fb*/
    qWarning("Testing SRCP FB messages...");

    /* FB SET*/
    qWarning("  SET...");
    FbMessage fbm = FbMessage(SrcpMessage::mtFbSet, (unsigned int)3,
            (unsigned int)2, (unsigned int)1);

    if (!checkSrcp08String("SET 3 FB 2 1", &fbm))
        returnvalue = false;

    qWarning("  GET...");
    /* FB GET*/
    fbm = FbMessage(SrcpMessage::mtFbGet, (unsigned int)3, (unsigned int)2);

    if (!checkSrcp08String("GET 3 FB 2", &fbm))
        returnvalue = false;

    qWarning("  INFO...");
    /* FB INFO*/
    fbm = FbMessage(SrcpMessage::mtFbInfo, SrcpMessage::code100,
            (unsigned int)3, (unsigned int)2, (unsigned int)1);
    
    if (!checkSrcp08String("100 INFO 3 FB 2 1", &fbm))
        returnvalue = false;


    /*SRCP Ga*/
    qWarning("Testing SRCP GA messages...");

    /*MM protocol*/
    /*  GA INIT*/
    qWarning("  INIT...");
    GaMessage gam = GaMessage(SrcpMessage::mtGaInit, (unsigned int)4,
            (unsigned int)2, GaMessage::proMM);

    if (!checkSrcp08String("INIT 4 GA 2 M", &gam))
        returnvalue = false;

    /*  GA TERM*/
    qWarning("  TERM...");
    gam = GaMessage(SrcpMessage::mtGaTerm, (unsigned int)5,
            (unsigned int)2);

    if (!checkSrcp08String("TERM 5 GA 2", &gam))
        returnvalue = false;


    qWarning("  SET...");
    /*  GA SET*/
    gam = GaMessage(SrcpMessage::mtGaSet, (unsigned int)1,
            (unsigned int)2,  GaMessage::proMM, (unsigned int)3,
            (unsigned int)4, (unsigned int)5);

    if (!checkSrcp08String("SET 1 GA 2 3 4 5", &gam))
        returnvalue = false;
    

    /*  GA GET*/
    qWarning("  GET...");
    gam = GaMessage(SrcpMessage::mtGaGet, (unsigned int)1,
            (unsigned int)2, GaMessage::proMM, (unsigned int)3);

    if (!checkSrcp08String("GET 1 GA 2 3", &gam))
        returnvalue = false;

    /*  GA INFO 100*/
    qWarning("  INFO...");
    gam = GaMessage(SrcpMessage::mtGaInfo, SrcpMessage::code100,
            (unsigned int)1, (unsigned int)2, GaMessage::proMM,
            (unsigned int)3, (unsigned int)4);

    if (!checkSrcp08String("100 INFO 1 GA 2 3 4", &gam))
        returnvalue = false;

    /*  GA INFO 101*/
    gam = GaMessage(SrcpMessage::mtGaInfo, SrcpMessage::code101,
            (unsigned int)1, (unsigned int)2, GaMessage::proMM);

    if (!checkSrcp08String("101 INFO 1 GA 2 M", &gam))
        returnvalue = false;

    /*  GA INFO 102*/
    gam = GaMessage(SrcpMessage::mtGaInfo, SrcpMessage::code102,
            (unsigned int)1, (unsigned int)2);

    if (!checkSrcp08String("102 INFO 1 GA 2", &gam))
        returnvalue = false;


    /* DCC protocol*/

    /*  GA INIT*/
    gam = GaMessage(SrcpMessage::mtGaInit, (unsigned int)1,
            (unsigned int)2, GaMessage::proDCC);

    if (!checkSrcp08String("INIT 1 GA 2 N", &gam))
        returnvalue = false;


    /*  GA SET*/
    gam = GaMessage(SrcpMessage::mtGaSet, (unsigned int)1,
            (unsigned int)2, GaMessage::proDCC, (unsigned int)3,
            (unsigned int)4, (unsigned int)5);

    if (!checkSrcp08String("SET 1 GA 2 3 4 5", &gam))
        returnvalue = false;
    

    /*  GA GET*/
    gam = GaMessage(SrcpMessage::mtGaGet, (unsigned int)1,
            (unsigned int)2, GaMessage::proDCC, (unsigned int)3);

    if (!checkSrcp08String("GET 1 GA 2 3", &gam))
        returnvalue = false;


    /*Server protocol*/

    /*  GA INIT*/
    gam = GaMessage(SrcpMessage::mtGaInit, (unsigned int)1,
            (unsigned int)2, GaMessage::proServer);

    if (!checkSrcp08String("INIT 1 GA 2 P", &gam))
        returnvalue = false;


    /*  GA SET*/
    gam = GaMessage(SrcpMessage::mtGaSet, (unsigned int)1,
            (unsigned int)2, GaMessage::proServer, (unsigned int)3,
            (unsigned int)4, (unsigned int)5);

    if (!checkSrcp08String("SET 1 GA 2 3 4 5", &gam))
        returnvalue = false;
    

    /*  GA GET*/
    gam = GaMessage(SrcpMessage::mtGaGet, (unsigned int)1,
            (unsigned int)2, GaMessage::proServer, (unsigned int)3);

    if (!checkSrcp08String("GET 1 GA 2 3", &gam))
        returnvalue = false;


    /*Selectrix protocol*/

    /*  GA INIT*/
    gam = GaMessage(SrcpMessage::mtGaInit, (unsigned int)1,
            (unsigned int)2, GaMessage::proSelectrix);

    if (!checkSrcp08String("INIT 1 GA 2 S", &gam))
        returnvalue = false;



    /*SRCP Gl*/
    qWarning("Testing SRCP GL messages...");
    
    /*MM protocol 1 function*/

    /*  GL INIT*/
    qWarning("  INIT...");
    /* INIT <bus> GL <addr> <prot> <protv> <maxsteps> <fcount>*/
    GlMessage* glm = new GlMessage(SrcpMessage::mtGlInit, 1, 2,
            GlMessage::proMM, 1, 14, 1);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("INIT 1 GL 2 M 1 14 1", glm))
        returnvalue = false;

    delete glm;

    unsigned int fmap = 0;
    /*  GL SET*/
    qWarning("  SET...");
    /*SET <bus> GL <addr> <drivemode> <currentv> <maxv> <f1> . . <fn>*/
    /* <type> <bus> <addr> <fcount> <drivemode> <currentv> <maxv> <fmap>*/
    glm = new GlMessage(SrcpMessage::mtGlSet, 1, 2, 1, 0, 20, 90, fmap);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("SET 1 GL 2 0 20 90 0", glm))
        returnvalue = false;

    delete glm;

    /*  GL GET*/
    qWarning("  GET...");
    glm = new GlMessage(SrcpMessage::mtGlGet, 1, 2);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("GET 1 GL 2", glm))
        returnvalue = false;

    delete glm;


    /*MM protocol 5 functions*/

    /*  GL INIT*/
    glm = new GlMessage(SrcpMessage::mtGlInit, 1, 2, GlMessage::proMM, 
            2, 14, 5);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("INIT 1 GL 2 M 2 14 5", glm))
        returnvalue = false;

    delete glm;

    /*  GL SET*/
    /*SET <bus> GL <addr> <drivemode> <currentv> <maxv> <f1> . . <fn>*/
    /*<type> <bus> <addr> <fcount> <drivemode> <currentv> <maxv> <fmap>*/
    fmap = (GlMessage::glF1 | GlMessage::glF2 |GlMessage::glF3
            |GlMessage::glF4);

    glm = new GlMessage(SrcpMessage::mtGlSet, 1, 2, 5, 1, 20, 90, fmap);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("SET 1 GL 2 1 20 90 0 1 1 1 1", glm))
        returnvalue = false;

    delete glm;

    /*  GL GET*/
    glm = new GlMessage(SrcpMessage::mtGlGet, 1, 2);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("GET 1 GL 2", glm))
        returnvalue = false;

    delete glm;


    /*DCC protocol 1 function*/

    /*  GL INIT
     INIT <bus> GL <addr> <prot> <protv> <maxsteps> <fcount>*/
    glm = new GlMessage(SrcpMessage::mtGlInit, 1, 2, GlMessage::proDCC,
            1, 14, 1);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("INIT 1 GL 2 N 1 14 1", glm))
        returnvalue = false;

    delete glm;

    /*  GL SET*/
    /* <type> <bus> <addr> <fcount> <drivemode> <currentv> <maxv> <fmap>*/
    glm = new GlMessage(SrcpMessage::mtGlSet, 1, 2, 1, 0, 20, 90, fmap);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("SET 1 GL 2 0 20 90 0", glm))
        returnvalue = false;

    delete glm;

    /*  GL GET*/
    glm = new GlMessage(SrcpMessage::mtGlGet, 1, 2);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("GET 1 GL 2", glm))
        returnvalue = false;

    delete glm;


    /*DCC protocol 5 functions*/

    /*  GL INIT*/
    glm = new GlMessage(SrcpMessage::mtGlInit, 1, 2, GlMessage::proDCC,
           2, 28, 5);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("INIT 1 GL 2 N 2 28 5", glm))
        returnvalue = false;

    delete glm;

    /*  GL SET*/
    fmap = (GlMessage::glF1 | GlMessage::glF2 |GlMessage::glF3
            |GlMessage::glF4);
    glm = new GlMessage(SrcpMessage::mtGlSet, 1, 2, 5, 0, 20, 90, fmap);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("SET 1 GL 2 0 20 90 0 1 1 1 1", glm))
        returnvalue = false;

    delete glm;
    
    /*  GL GET*/
    glm = new GlMessage(SrcpMessage::mtGlGet, 2, 2);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("GET 2 GL 2", glm))
        returnvalue = false;

    delete glm;


    /*DCC protocol 9 functions*/

    /*  GL INIT*/
    glm = new GlMessage(SrcpMessage::mtGlInit, 1, 2, GlMessage::proDCC,
            2, 128, 9);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("INIT 1 GL 2 N 2 128 9", glm))
        returnvalue = false;

    delete glm;

    /*  GL SET*/
    fmap = (GlMessage::glF1 | GlMessage::glF2 | GlMessage::glF3
            | GlMessage::glF4 | GlMessage::glF5 | GlMessage::glF6
            | GlMessage::glF7 | GlMessage::glF8);
    glm = new GlMessage(SrcpMessage::mtGlSet, 1, 2, 9, 0, 20, 90, fmap);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("SET 1 GL 2 0 20 90 0 1 1 1 1 1 1 1 1", glm))
        returnvalue = false;

    delete glm;

    /*  GL GET*/
    glm = new GlMessage(SrcpMessage::mtGlGet, 1, 2);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("GET 1 GL 2", glm))
        returnvalue = false;

    delete glm;


    /*DCC protocol 13 functions*/

    /*  GL INIT*/
    glm = new GlMessage(SrcpMessage::mtGlInit, 1, 2, GlMessage::proDCC,
            2, 128, 13);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("INIT 1 GL 2 N 2 128 13", glm))
        returnvalue = false;

    delete glm;

    /*  GL SET*/
    fmap = (GlMessage::glF1 | GlMessage::glF2 | GlMessage::glF3
            | GlMessage::glF4 | GlMessage::glF9 | GlMessage::glF10
            | GlMessage::glF11 | GlMessage::glF12);

    glm = new GlMessage(SrcpMessage::mtGlSet, 1, 2, 13, 0, 20, 90, fmap);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("SET 1 GL 2 0 20 90 0 1 1 1 1 0 0 0 0 1 1 1 1", glm))
        returnvalue = false;

    delete glm;

    /*  GL GET*/
    glm = new GlMessage(SrcpMessage::mtGlGet, 1, 2);
    if (glm == NULL)
        return false;

    if (!checkSrcp08String("GET 1 GL 2", glm))
        returnvalue = false;

    delete glm;

    /* INFO*/
    qWarning("  INFO...");
    qWarning("-> missing");

    /*SRCP GM*/
    qWarning("Testing SRCP GM messages...");
    qWarning("-> missing");

    /*SRCP Lock*/
    qWarning("Testing SRCP LOCK messages...");
    
    /* GET*/
    qWarning("  GET...");

    /*   GA*/
    LockMessage lm = LockMessage(SrcpMessage::mtLockGet, 3, SrcpMessage::devGa,
            (unsigned int)128);

    if (!checkSrcp08String("GET 3 LOCK GA 128", &lm))
        returnvalue = false;

    /*   GL*/
    lm = LockMessage(SrcpMessage::mtLockGet, 3, SrcpMessage::devGl,
            (unsigned int)107);

    if (!checkSrcp08String("GET 3 LOCK GL 107", &lm))
        returnvalue = false;

    /* SET*/
    qWarning("  SET...");

    /*   GA*/
    lm = LockMessage(SrcpMessage::mtLockSet, 3, SrcpMessage::devGa,
            (unsigned int)168, (unsigned int)13);

    if (!checkSrcp08String("SET 3 LOCK GA 168 13", &lm))
        returnvalue = false;


    /* TERM*/
    qWarning("  TERM...");

    /*   GA*/
    lm = LockMessage(SrcpMessage::mtLockTerm, (unsigned int)3,
            SrcpMessage::devGa, (unsigned int)223);

    if (!checkSrcp08String("TERM 3 LOCK GA 223", &lm))
        returnvalue = false;

    /* INFO*/
    qWarning("  INFO...");

    /*   100 GA*/
    lm = LockMessage(SrcpMessage::mtLockInfo, SrcpMessage::code100,
            (unsigned int)3, SrcpMessage::devGa, 223, 20, 6);

    if (!checkSrcp08String("100 INFO 3 LOCK GA 223 20 6", &lm))
        returnvalue = false;

    /*   101 GA*/
    lm = LockMessage(SrcpMessage::mtLockInfo, SrcpMessage::code101,
            (unsigned int)3, SrcpMessage::devGa, 223);

    if (!checkSrcp08String("101 INFO 3 LOCK GA 223", &lm))
        returnvalue = false;

    /*   102 GL*/
    lm = LockMessage(SrcpMessage::mtLockInfo, SrcpMessage::code102,
            (unsigned int)5, SrcpMessage::devGl, 106);

    if (!checkSrcp08String("102 INFO 5 LOCK GL 106", &lm))
        returnvalue = false;


    /*SRCP Power*/
    qWarning("Testing SRCP POWER messages...");

    /* INIT*/
    qWarning("  INIT...");
    PowerMessage pm = PowerMessage(SrcpMessage::mtPowerInit,
            (unsigned int)1);

    if (!checkSrcp08String("INIT 1 POWER", &pm))
        returnvalue = false;

    /* TERM*/
    qWarning("  TERM...");
    pm = PowerMessage(SrcpMessage::mtPowerTerm,
            (unsigned int)2);

    if (!checkSrcp08String("TERM 2 POWER", &pm))
        returnvalue = false;


    /* SET off*/
    qWarning("  SET...");
    pm = PowerMessage(SrcpMessage::mtPowerSet,
            (unsigned int)1, false);

    if (!checkSrcp08String("SET 1 POWER OFF", &pm))
        returnvalue = false;

    pm = PowerMessage(SrcpMessage::mtPowerSet,
            (unsigned int)2, false);

    if (!checkSrcp08String("SET 2 POWER OFF", &pm))
        returnvalue = false;

    /* SET on*/
    pm = PowerMessage(SrcpMessage::mtPowerSet,
            (unsigned int)1, true);

    if (!checkSrcp08String("SET 1 POWER ON", &pm))
        returnvalue = false;

    pm = PowerMessage(SrcpMessage::mtPowerSet,
            (unsigned int)2, true);

    if (!checkSrcp08String("SET 2 POWER ON", &pm))
        returnvalue = false;

    /* GET*/
    qWarning("  GET...");
    pm = PowerMessage(SrcpMessage::mtPowerGet, (unsigned int)1);

    if (!checkSrcp08String("GET 1 POWER", &pm))
        returnvalue =  false;

    pm = PowerMessage(SrcpMessage::mtPowerGet, (unsigned int)2);

    if (!checkSrcp08String("GET 2 POWER", &pm))
        returnvalue =  false;

    /* INFO off*/
    qWarning("  INFO...");
    pm = PowerMessage(SrcpMessage::mtPowerInfo, SrcpMessage::code100,
            (unsigned int)1, false);

    if (!checkSrcp08String("100 INFO 1 POWER OFF", &pm))
        returnvalue =  false;

    pm = PowerMessage(SrcpMessage::mtPowerInfo, SrcpMessage::code100,
            (unsigned int)2, false);

    if (!checkSrcp08String("100 INFO 2 POWER OFF", &pm))
        returnvalue =  false;

    /* INFO on*/
    pm = PowerMessage(SrcpMessage::mtPowerInfo, SrcpMessage::code100,
            (unsigned int)1, true);

    if (!checkSrcp08String("100 INFO 1 POWER ON", &pm))
        returnvalue =  false;

    pm = PowerMessage(SrcpMessage::mtPowerInfo, SrcpMessage::code100,
            (unsigned int)2, true);

    if (!checkSrcp08String("100 INFO 2 POWER ON", &pm))
        returnvalue =  false;


    /*SRCP Server*/
    qWarning("Testing SRCP SERVER messages...");

    /*Get*/
    qWarning("  GET...");
    ServerMessage svm = ServerMessage(SrcpMessage::mtServerGet);

    if (!checkSrcp08String("GET 0 SERVER", &svm))
        returnvalue =  false;



    /*Reset*/
    qWarning("  RESET...");
    svm = ServerMessage(SrcpMessage::mtServerReset);

    if (!checkSrcp08String("RESET 0 SERVER", &svm))
        returnvalue =  false;


    /*Term*/
    qWarning("  TERM...");
    svm = ServerMessage(SrcpMessage::mtServerTerm);

    if (!checkSrcp08String("TERM 0 SERVER", &svm))
        returnvalue =  false;

    
    /*Info*/
    qWarning("  INFO...");
    /* RUNNING*/
    svm = ServerMessage(SrcpMessage::mtServerInfo, SrcpMessage::code100,
            ServerMessage::ssRunning);

    if (!checkSrcp08String("100 INFO 0 SERVER RUNNING", &svm))
        returnvalue =  false;


    /* RESETTING*/
    svm = ServerMessage(SrcpMessage::mtServerInfo, SrcpMessage::code100,
            ServerMessage::ssResetting);

    if (!checkSrcp08String("100 INFO 0 SERVER RESETTING", &svm))
        returnvalue =  false;


    /* TERMINATING*/
    svm = ServerMessage(SrcpMessage::mtServerInfo, SrcpMessage::code100,
            ServerMessage::ssTerminating);

    if (!checkSrcp08String("100 INFO 0 SERVER TERMINATING", &svm))
        returnvalue =  false;


    /* TIME init*/
    svm = ServerMessage(SrcpMessage::mtServerInfo, SrcpMessage::code101);

    if (!checkSrcp08String("101 INFO 0 SERVER", &svm))
        returnvalue =  false;


    /* Server termiation*/
    svm = ServerMessage(SrcpMessage::mtServerInfo, SrcpMessage::code102);

    if (!checkSrcp08String("102 INFO 0 SERVER", &svm))
        returnvalue =  false;


 

    /*SRCP Session*/
    qWarning("Testing SRCP SESSION messages...");

    /*Get*/
    qWarning("  GET...");
    SessionMessage sem = SessionMessage(SrcpMessage::mtSessionGet, 3);

    if (!checkSrcp08String("GET 0 SESSION 3", &sem))
        returnvalue = false;


    /*Term*/
    qWarning("  TERM...");

    /*without id*/
    sem = SessionMessage(SrcpMessage::mtSessionTerm);

    if (!checkSrcp08String("TERM 0 SESSION", &sem))
        returnvalue = false;



    /*with id*/
    sem = SessionMessage(SrcpMessage::mtSessionTerm, 3);

    if (!checkSrcp08String("TERM 0 SESSION 3", &sem))
        returnvalue = false;


    /*Info*/
    qWarning("  INFO...");

    /*100*/
    sem = SessionMessage(SrcpMessage::mtSessionInfo,
            SrcpMessage::code100, 3);

    if (!checkSrcp08String("100 INFO 0 SESSION 3", &sem))
        returnvalue = false;


    /*101*/
    sem = SessionMessage(SrcpMessage::mtSessionInfo,
            SrcpMessage::code101, 4);

    if (!checkSrcp08String("101 INFO 0 SESSION 4", &sem))
        returnvalue = false;


    /*102*/
    sem = SessionMessage(SrcpMessage::mtSessionInfo,
            SrcpMessage::code102, 5);

    if (!checkSrcp08String("102 INFO 0 SESSION 5", &sem))
        returnvalue = false;




    /*SRCP Service mode*/
    qWarning("Testing SRCP SM messages...");

    /* INIT*/
    qWarning("  INIT...");
    SmMessage smm = SmMessage(SrcpMessage::mtSmInit, (unsigned int)3,
           SmMessage::smpNmra);

    if (!checkSrcp08String("INIT 3 SM NMRA", &smm))
        returnvalue = false;

    /* TERM*/
    qWarning("  TERM...");
    smm = SmMessage(SrcpMessage::mtSmTerm, (unsigned int)5);

    if (!checkSrcp08String("TERM 5 SM", &smm))
        returnvalue = false;

    /* SET*/
    qWarning("  SET...");

    /*    CV*/
    smm = SmMessage(SrcpMessage::mtSmSet, (unsigned int)5,
            (unsigned int)127, SmMessage::smtCv, 63, 7);

    if (!checkSrcp08String("SET 5 SM 127 CV 63 7", &smm))
        returnvalue = false;

    /*    REG*/
    smm = SmMessage(SrcpMessage::mtSmSet, (unsigned int)5,
            (unsigned int)126, SmMessage::smtReg, 89, 6);

    if (!checkSrcp08String("SET 5 SM 126 REG 89 6", &smm))
        returnvalue = false;

    /*    PAGE*/
    smm = SmMessage(SrcpMessage::mtSmSet, (unsigned int)12,
            (unsigned int)126, SmMessage::smtPage, 17, 8);

    if (!checkSrcp08String("SET 12 SM 126 PAGE 17 8", &smm))
        returnvalue = false;

    /*    CVBIT*/
    smm = SmMessage(SrcpMessage::mtSmSet, (unsigned int)12,
            (unsigned int)126, SmMessage::smtCvbit, 17, 3, 8);

    if (!checkSrcp08String("SET 12 SM 126 CVBIT 17 3 8", &smm))
        returnvalue = false;


    /* VERIFY*/
    qWarning("  VERIFY...");

    /*    CV*/
    smm = SmMessage(SrcpMessage::mtSmVerify, (unsigned int)5,
            (unsigned int)127, SmMessage::smtCv, 63, 7);

    if (!checkSrcp08String("VERIFY 5 SM 127 CV 63 7", &smm))
        returnvalue = false;

    /*    REG*/
    smm = SmMessage(SrcpMessage::mtSmVerify, (unsigned int)5,
            (unsigned int)126, SmMessage::smtReg, 89, 6);

    if (!checkSrcp08String("VERIFY 5 SM 126 REG 89 6", &smm))
        returnvalue = false;

    /*    PAGE*/
    smm = SmMessage(SrcpMessage::mtSmVerify, (unsigned int)12,
            (unsigned int)126, SmMessage::smtPage, 17, 8);

    if (!checkSrcp08String("VERIFY 12 SM 126 PAGE 17 8", &smm))
        returnvalue = false;

    /*    CVBIT*/
    smm = SmMessage(SrcpMessage::mtSmVerify, (unsigned int)12,
            (unsigned int)126, SmMessage::smtCvbit, 17, 3, 8);

    if (!checkSrcp08String("VERIFY 12 SM 126 CVBIT 17 3 8", &smm))
        returnvalue = false;


    /* GET*/
    qWarning("  GET...");

    /*    CV*/
    smm = SmMessage(SrcpMessage::mtSmGet, (unsigned int)5,
            (unsigned int)127, SmMessage::smtCv, 63);

    if (!checkSrcp08String("GET 5 SM 127 CV 63", &smm))
        returnvalue = false;

    /*    REG*/
    smm = SmMessage(SrcpMessage::mtSmGet, (unsigned int)5,
            (unsigned int)126, SmMessage::smtReg, 89);

    if (!checkSrcp08String("GET 5 SM 126 REG 89", &smm))
        returnvalue = false;

    /*    PAGE*/
    smm = SmMessage(SrcpMessage::mtSmGet, (unsigned int)12,
            (unsigned int)126, SmMessage::smtPage, 17);

    if (!checkSrcp08String("GET 12 SM 126 PAGE 17", &smm))
        returnvalue = false;

    /*    CVBIT*/
    smm = SmMessage(SrcpMessage::mtSmGet, (unsigned int)12,
            (unsigned int)126, SmMessage::smtCvbit, 17, 3, 0);

    if (!checkSrcp08String("GET 12 SM 126 CVBIT 17 3", &smm))
        returnvalue = false;

    /* INFO*/
    qWarning("  INFO...");
    qWarning("  missing.");


    /*SRCP Time*/
    qWarning("Testing SRCP TIME messages...");

    /* INIT*/
    qWarning("  INIT...");
    TimeMessage tm = TimeMessage(SrcpMessage::mtTimeInit,
            (unsigned int)3, (unsigned int)4);

    if (!checkSrcp08String("INIT 0 TIME 3 4", &tm))
        returnvalue = false;

    /* TERM*/
    qWarning("  TERM...");
    tm = TimeMessage(SrcpMessage::mtTimeTerm);

    if (!checkSrcp08String("TERM 0 TIME", &tm))
        returnvalue = false;


    /* GET*/
    qWarning("  GET...");
    tm = TimeMessage(SrcpMessage::mtTimeGet);

    if (!checkSrcp08String("GET 0 TIME", &tm))
        returnvalue = false;

    /* SET*/
    qWarning("  SET...");
    tm = TimeMessage(SrcpMessage::mtTimeSet, (unsigned int)50,
            (unsigned int)16, (unsigned int)58, (unsigned int)47);

    if (!checkSrcp08String("SET 0 TIME 50 16 58 47", &tm))
        returnvalue = false;

    /* WAIT*/
    qWarning("  WAIT...");
    tm = TimeMessage(SrcpMessage::mtTimeWait, (unsigned int)2,
            (unsigned int)5, (unsigned int)23, (unsigned int)34);

    if (!checkSrcp08String("WAIT 0 TIME 2 5 23 34", &tm))
        returnvalue = false;

    /* INFO 100*/
    qWarning("  INFO...");
    tm = TimeMessage(SrcpMessage::mtTimeInfo, SrcpMessage::code100,
            (unsigned int)2, (unsigned int)3, (unsigned int)4,
            (unsigned int)5);

    if (!checkSrcp08String("100 INFO 0 TIME 2 3 4 5", &tm))
        returnvalue = false;

    /* INFO 101*/
    tm = TimeMessage(SrcpMessage::mtTimeInfo, SrcpMessage::code101,
            (unsigned int)2, (unsigned int)3);

    if (!checkSrcp08String("101 INFO 0 TIME 2 3", &tm))
        returnvalue = false;

    /* INFO 102*/
    tm = TimeMessage(SrcpMessage::mtTimeInfo, SrcpMessage::code102);

    if (!checkSrcp08String("102 INFO 0 TIME", &tm))
        returnvalue = false;



    return returnvalue;
}


/*create and run console application*/
int main(int argc, char* argv[])
{
   QApplication testApp(argc, argv, false);

   bool success = runSrcpTest();

   return (success) ? 0 : 1;
}

